// Simple display of a procedurally generated 2D image

#define MAIN
#include "MicroGlut.h"
#include "GL_utilities.h"
#include "LittleOBJLoader.h"
	// uses framework Cocoa
	// uses framework OpenGL

// Globals
// Data would normally be read from files
GLfloat vertices[] = {	-1.0f,-1.0f,0.0f,
							-1.0f,1.0f,0.0f,
							1.0f,1.0f,0.0f,
							1.0f,-1.0f,0.0f};
GLfloat texcoord[] = {	0.0f, 1.0f,
						0.0f, 0.0f,
						1.0f, 0.0f,
						1.0f, 1.0f};
GLuint indices[] = {0,1,3, 3,1,2};


#define kTextureSize 512

// Texture data:
GLubyte ptex[kTextureSize][kTextureSize][3];

// Example: Radial pattern.
void maketexture()
{
	int x, y;
	float fx, fy, fxo, fyo;
	char val;
	
	for (x = 0; x < kTextureSize; x++)
	for (y = 0; y < kTextureSize; y++)
	{
		ptex[x][y][0] = x*2;
		ptex[x][y][1] = y*2;
		ptex[x][y][2] = 128;
		
		fx = (float)(x-kTextureSize/2.)/kTextureSize*2.;
		fy = (float)(y-kTextureSize/2.)/kTextureSize*2.;
		fxo = sqrt(fx*fx+fy*fy);
		fyo = sqrt(fx*fx+fy*fy);
		fxo = cos(fxo * 20);
		fyo = sin(fyo * 20);
		if (fxo > 1.0) fxo = 1;
		if (fxo < -1.0) fxo = -1.0;
		if (fyo > 1.0) fyo = 1.0;
		if (fyo < -1.0) fyo = -1.0;
		ptex[x][y][0] = fxo * 127 + 127;
		ptex[x][y][1] = fyo * 127 + 127;
		ptex[x][y][2] = 128;
	}
}

// Texture reference
GLuint texid;
// Model (just a quad)
Model *m;
// Switch between CPU and shader generation
int flag = 0;

// Reference to shader program
GLuint program;

void init(void)
{
	// GL inits
	glClearColor(0.2,0.2,0.5,0);
	glEnable(GL_DEPTH_TEST);
	printError("GL inits");

	// Load and compile shader
	program = loadShaders("minimal.vert", "procedural.frag");
	glUseProgram(program);
	printError("init shader");
	
	// Upload geometry to the GPU:
	m = LoadDataToModel((vec3 *)vertices, NULL, (vec2 *)texcoord, NULL, indices, 4, 6);

	// Texture unit
	glUniform1i(glGetUniformLocation(program, "tex"), 0); // Texture unit 0

	glUniform1i(glGetUniformLocation(program, "flag"), 0); // shader generation off

	maketexture();

// Upload texture
	glGenTextures(1, &texid); // texture id
	glBindTexture(GL_TEXTURE_2D, texid);
	printError("init texture 1");
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	printError("init texture 2");
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR); //  
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR); // 
	printError("init texture 3");
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, kTextureSize, kTextureSize, 0, GL_RGB, GL_UNSIGNED_BYTE, ptex);

	// End of upload of geometry
	printError("init arrays");
}

// Switch on any key
void key(unsigned char key, int x, int y)
{
	flag = !flag;
	glUniform1i(glGetUniformLocation(program, "flag"), flag); // shader generation off
	printf("Changed flag to %d\n", flag);
	glutPostRedisplay();
}

void display(void)
{
	printError("pre display");
	
	// clear the screen
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	DrawModel(m, program, "inPosition", NULL, "inTexCoord");
	
	printError("display");
	
	glFlush();
}

int main(int argc, char *argv[])
{
	glutInit(&argc, argv);
	glutInitWindowSize(kTextureSize, kTextureSize);
	glutInitContextVersion(3, 2);
	glutCreateWindow ("texture generation example");
	glutDisplayFunc(display); 
	glutKeyboardFunc(key);
	init ();
	glutMainLoop();
}
